/******************************************************************************
*
* TI Clock Installer, programmed by Johannes Zeppenfeld
* adapted from the tigcc clock tutorial by Thomas Nussbaumer
*
*******************************************************************************
* build this program by typing: tigcc -O2 tsrclock.c
*
* For the original TICT tutorial this code was based off, visit:
* http://tigcc.ticalc.org/tut/
* ...tsrclock.html
* ...tsr_source.txt
*******************************************************************************
* You may use this code however you like, but please give credit to those who
* wrote it.
* Other than that, this code is e-note-ware: if you use it, please send me an
* email telling me about your project and how you integrated the clock into it.
* I love to hear what others use programs I write for.
* Johannes Zeppenfeld (jzeppenfeld@excite.com)
******************************************************************************/

#include <tigcclib.h>

int _ti89;					// produce code for TI89 only

#define SIZE_OF_TSR sizeof(tsr)
//#define DEBUG				// Used to find TSR offset from time structure

#ifndef TRAP_5
#define TRAP_5 0x94
#endif

//----------------------------------------------------------
// the TSR clock hook (see project ClkHook for its source code)
// compiled with: tigcc -O3 -outputbin clkhook.c
// and converted to an array with the ttbin2hex tool
// use the included make file for this purpose
//----------------------------------------------------------
#include "array.h"
//=============================================================================
//	finds and returns address of clock data structure
//=============================================================================
char *ClockFind(void) {
  unsigned long loc = peek_l(TRAP_5);
	if(loc>=(unsigned long)0x200000) return NULL;
	return (char *)(loc-0x40000); //if found, return the location
}
//=============================================================================
// installs TSR program
//
// returns 1 (or structure offset(!= 0) in debug) if successfully installed
//=============================================================================
long InstallTSR(void) {
    unsigned char* mem = HeapAllocPtr(SIZE_OF_TSR); // +no size bytes
    //---------------------------------------------------    
    // in case of a "cannot allocate enough memory" error
    // we will immediately return
    //---------------------------------------------------
    if (!mem) return 0;
    //-----------------------------------------------------
    // copy the program itself from the tsr array
    //-----------------------------------------------------
    memcpy(mem,tsr,SIZE_OF_TSR);
    
    //--------------------------------------------------------
    // relocate program, bypass some protections introduced in 
    // AMS 2.xx and execute _main routine of TSR
    //--------------------------------------------------------
    EX_patch(mem + 0x40000, mem + SIZE_OF_TSR - 2 + 3 - 2);  
    enter_ghost_space();
    ASM_call(mem + 0x40000);

#ifdef DEBUG
		printf_xy(50,50,"%li",(long)ClockFind()-(long)mem);
		ngetchx();
#endif
    return 1;
}


//=============================================================================
// removes TSR program
//=============================================================================
void RemoveTSR(unsigned char* tsr_address) {
    //------------------------------------------------
    // bypass some protections introduced in 
    // AMS 2.xx and execute _main routine of TSR again
    //------------------------------------------------
    enter_ghost_space();
    ASM_call(tsr_address + 0x40000);
    //--------------------------------------------------
    // free the buffer allocated by InstallTSR()
    //--------------------------------------------------
    HeapFreePtr(tsr_address); 
}
//=============================================================================
// as usual: a main entry point ...
//=============================================================================
void _main(void) { 
	unsigned char* tsr_address;
  long i;
	//----------------------------------------------------
	// try to find an already installed version of the TSR
	//
	// if the clock asserted its presence
	// the TSR is already installed -> remove it
	//-------------------------------------------
	if((tsr_address = ClockFind())) {
		RemoveTSR(tsr_address-18);				//Clockfind returns a pointer to the time structure
		ST_showHelp("TSR clock removed");	//RemoveTSR needs a pointer to the start of the TSR code
	} else {														//which is offset by 18
		if (!(i=InstallTSR())) {
			ST_showHelp("cannot install TSR clock");
		} else {
			ST_showHelp("TSR clock installed");
		}
	}
}
